import React from 'react';
import { Stage, Layer, Group, Rect, Line, Shape } from 'react-konva';

import { includeNewMask } from '../../utils/canvas';

const HighlightShape = ({ width, height, masks, currentMask }) => {
  return (
    <Shape
      x={0}
      y={0}
      width={width}
      height={height}
      sceneFunc={(ctx, shape) => {
        ctx.beginPath();
        ctx.moveTo(0, 0);
        ctx.lineTo(width, 0);
        ctx.lineTo(width, height);
        ctx.lineTo(0, height);
        ctx.lineTo(0, 0);
        ctx.closePath();

        masks.forEach(maskPoints => {
          const startPoint = maskPoints[0];
          ctx.moveTo(startPoint[0], startPoint[1]);

          maskPoints.forEach(point => {
            ctx.lineTo(point[0], point[1]);
          });
          ctx.closePath();
        });

        if (currentMask.startX != -1 && currentMask.endX !== -1) {
          ctx.moveTo(currentMask.startX, currentMask.startY);
          ctx.lineTo(currentMask.startX, currentMask.endY);
          ctx.lineTo(currentMask.endX, currentMask.endY);
          ctx.lineTo(currentMask.endX, currentMask.startY);
          ctx.lineTo(currentMask.startX, currentMask.startY);
          ctx.closePath();
        }

        ctx.fillStyle = 'rgba(0,0,0,0.3)';
        ctx.strokeStyle = 'rgba(0.5,0.5,0.5,0.5)';
        ctx.lineWidth = 1;
        ctx.fill();
        ctx.stroke();
      }}
    />
  );
};

class FeedbackDrawer extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      isDrawing: false,
      highlightMasks: [
        // [[100, 100], [100, 400], [200, 400], [200, 100], [100, 100]]
      ],
      currentStartX: -1,
      currentStartY: -1,
      currentEndX: -1,
      currentEndY: -1
    };

    this.onMouseDown = this.onMouseDown.bind(this);
    this.onMouseUp = this.onMouseUp.bind(this);
    this.onMouseMove = this.onMouseMove.bind(this);
  }

  onMouseDown(event) {
    if (this.props.isCanvasActive && !this.state.isDrawing) {
      // this should be triggerd via onMouseMove when currentStartX - event.evt.offsetX > 20
      this.setState({
        isDrawing: true,
        currentStartX: event.evt.offsetX,
        currentStartY: event.evt.offsetY
      });
    }
  }

  onMouseUp(event) {
    if (this.props.isCanvasActive && this.state.isDrawing) {
      this.setState(prevState => {
        const { highlightMasks, currentStartX, currentStartY } = prevState;

        const startX =
          currentStartX >= event.evt.offsetX
            ? currentStartX
            : event.evt.offsetX;
        const startY =
          currentStartY >= event.evt.offsetY
            ? currentStartY
            : event.evt.offsetY;
        const endX =
          currentStartX < event.evt.offsetX ? currentStartX : event.evt.offsetX;
        const endY =
          currentStartY < event.evt.offsetY ? currentStartY : event.evt.offsetY;

        const newHighlightMask = [
          [startX, startY],
          [startX, endY],
          [endX, endY],
          [endX, startY],
          [startX, startY]
        ];

        const newHighlightMasks = includeNewMask({
          highlightMasks,
          newHighlightMask
        });

        return {
          isDrawing: false,
          currentStartX: -1,
          currentStartY: -1,
          currentEndX: -1,
          currentEndY: -1,
          highlightMasks: newHighlightMasks
        };
      });
    }
  }

  onMouseMove(event) {
    if (this.props.isCanvasActive && this.state.isDrawing) {
      this.setState({
        currentEndX: event.evt.offsetX,
        currentEndY: event.evt.offsetY
      });
    }
  }

  render() {
    var width = document.body.scrollWidth;
    var height = document.body.scrollHeight;

    const {
      highlightMasks,
      currentStartX,
      currentStartY,
      currentEndX,
      currentEndY
    } = this.state;

    return (
      <Stage
        style={{
          position: 'fixed',
          top: 0,
          left: 0,
          zIndex: 1
        }}
        width={width}
        height={height}
      >
        <Layer>
          <Group>
            {/* Dummy Background
            <Line
              points={[173, 140, 340, 23, 500, 109]}
              stroke="red"
              strokeWidth={10}
              fill="blue"
              tension={11}
              scale={{ x: 0.5, y: 0.5 }}
              x={0}
              y={0}
              closed={true}
            /> */}

            <HighlightShape
              masks={highlightMasks}
              currentMask={{
                startX: currentStartX,
                startY: currentStartY,
                endX: currentEndX,
                endY: currentEndY
              }}
              width={width}
              height={height}
            />

            <Rect
              id="fake-background"
              x={0}
              y={0}
              width={width}
              height={height}
              onMouseDown={this.onMouseDown}
              onMouseUp={this.onMouseUp}
              onMouseMove={this.onMouseMove}
            />
          </Group>
        </Layer>
      </Stage>
    );
  }
}

export default FeedbackDrawer;
